from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class HouseNumber(DarBaseModel):
    access_addresses_designation: Optional[str] = Field(
        None,
        alias="adgangsadressebetegnelse",
        description="EAID_EDCC14FC_63ED_4ebe_8B52_3E406831A148",
    )
    access_point: Optional[str] = Field(
        None,
        alias="adgangspunkt",
        description="EAID_E3A37E0C_D0D8_4c30_BBD5_2F26F7A24591",
    )
    house_number_direction: Optional[str] = Field(
        None,
        alias="husnummerretning",
        description="EAID_1A906922_EFAC_4182_AA8C_08332D52C6EE",
    )
    house_number_text: Optional[str] = Field(
        None,
        alias="husnummertekst",
        description="EAID_83A4C230_5C5A_4ffb_B136_0315DD9F8DBE",
    )
    waypoint: Optional[str] = Field(
        None,
        alias="vejpunkt",
        description="EAID_F6BAEBD6_0306_4641_9DEA_C1441D986AFB",
    )
    piece_of_land: Optional[str] = Field(
        None,
        alias="jordstykke",
        description="EAID_dst052F4B_D60C_4ccd_BAE5_6BCED78FE40C",
    )
    placed_on_a_temporary_plot_of_land: Optional[str] = Field(
        None,
        alias="placeretPåForeløbigtJordstykke",
        description="EAID_dstC82C82_CC9D_4d7d_9B8A_8F4A1ADFB5AE",
    )
    geo_danmark_building: Optional[str] = Field(
        None,
        alias="geoDanmarkBygning",
        description="EAID_dst3C28C5_D640_4bc0_9014_FDC7CBF30581",
    )
    access_to_building: Optional[str] = Field(
        None,
        alias="adgangTilBygning",
        description="EAID_dst3E4349_6115_4191_AF4F_243CCE561C41",
    )
    access_to_technical_facilities: Optional[str] = Field(
        None,
        alias="adgangTilTekniskAnlæg",
        description="EAID_dst79A4C1_23B2_4c0c_85CC_60C6E2EA5AE0",
    )
    road_midpoint: Optional[str] = Field(
        None,
        alias="vejmidte",
        description="EAID_dstC9842D_CF82_4dc5_8114_3FF7AEF90C17",
    )
    voting_area: Optional[str] = Field(
        None,
        alias="afstemningsområde",
        description="EAID_dstB6088D_EE9E_451c_8070_088AA4520FA6",
    )
    municipal_division: Optional[str] = Field(
        None,
        alias="kommuneinddeling",
        description="EAID_dst714AB4_FE21_446f_BEE5_7AE65691A45B",
    )
    parish_council_voting_area: Optional[str] = Field(
        None,
        alias="menighedsrådsafstemningsområde",
        description="EAID_dst08DF20_7055_4542_8D3D_F8FCCD9A7125",
    )
    parish_division: Optional[str] = Field(
        None,
        alias="sogneinddeling",
        description="EAID_dst0864F0_E380_4aa6_A644_25974107FC47",
    )
    supplermentary_city_name: Optional[str] = Field(
        None,
        alias="supplerendeBynavn",
        description="EAID_dstF5C960_23DB_454c_A8A8_41B231DF61C7",
    )
    named_road: Optional[str] = Field(
        None,
        alias="navngivenVej",
        description="EAID_dst74C4B5_0F0F_4a51_B317_BA74938B605F",
    )
    postal_code: Optional[str] = Field(
        None,
        alias="postnummer",
        description="EAID_dst3F5BA7_EB6C_47a2_AC4B_90C72D4F3585",
    )



import logging
from pathlib import Path
import sys
from zipfile import ZipFile

from dar_etl.config import Config
from dar_etl.consumer import NewDarFileNotificationConsumer
from dar_etl.parsing.parser import DarParser
from dar_etl.publisher import DarEntryPublisher


class DarEtl:
    def __init__(self, config: Config) -> None:
        self.publisher = DarEntryPublisher(config=config)
        self.consumer = NewDarFileNotificationConsumer(config=config)
        self.json_directory = config.app.json_directory
        self.parser = DarParser()

    def run(self) -> None:
        for filepath in self.consumer.consume():
            json_file = self._unzip(filepath=filepath)
            for entry in self.parser.parse(filepath=json_file):
                self.publisher.publish(metadata=entry)

    def _unzip(self, filepath: Path) -> Path:
        logging.info(f"[EXTRACTING] {filepath.name}")
        member = filepath.name.replace(".zip", ".json")
        json_file = self.json_directory.joinpath(member)
        if json_file.exists():
            logging.info(f"[EXTRACTED] {filepath.name}")
            return json_file
        with ZipFile(file=filepath, mode="r") as dar_zip_file:
            dar_zip_file.extract(member=member, path=self.json_directory)
        logging.info(f"[EXTRACTED] {filepath.name}")
        return json_file


def main() -> None:
    config = Config()
    logging.basicConfig(stream=sys.stdout, level=config.log.level)
    dar_etl = DarEtl(config=config)
    dar_etl.run()


if __name__ == "__main__":
    main()



from pathlib import Path

from pydantic import BaseSettings

from dar_etl.version import __version__


class BaseConfig(BaseSettings):
    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"


class KafkaConfig(BaseConfig):
    host: str
    port: int
    topic: str

    class Config:
        env_prefix = "kafka_"


class LogConfig(BaseConfig):
    level: str = "INFO"

    class Config:
        env_prefix = "log_"


class AppConfig(BaseConfig):
    input_directory: Path
    json_directory: Path


class Config:
    def __init__(self) -> None:
        self.version: str = __version__
        self.kafka = KafkaConfig()
        self.app = AppConfig()
        self.log = LogConfig()



from enum import Enum


class Root(Enum):
    addresses = "AdresseList"
    address_points = "AdressepunktList"
    house_numbers = "HusnummerList"
    named_roads = "NavngivenVejList"
    named_road_municipal_parts = "NavngivenVejKommunedelList"
    named_road_postal_codes = "NavngivenVejPostnummerList"
    named_road_supplementary_city_names = "NavngivenVejSupplerendeBynavnList"
    postal_codes = "PostnummerList"
    supplementary_city_names = "SupplerendeBynavnList"



from datetime import datetime
from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class NamedRoad(DarBaseModel):
    administreres_af_kommune: Optional[str] = Field(
        None,
        alias="administreresAfKommune",
        description="EAID_31D3BE04_2F70_4092_8381_5517A5543634",
    )
    description: Optional[str] = Field(None, description="EAID_C132D872_7F07_4ebf_BC6B_37049F2E5EB8")
    pronounced_street_name: Optional[str] = Field(
        None,
        alias="udtaltVejnavn",
        description="EAID_788F8478_A747_412d_9DE6_FEE0DB4C1BFA",
    )
    road_address_name: Optional[str] = Field(None, description="EAID_9621DD9D_50E3_4426_AB5A_A5E43DED4B05")
    street_name: Optional[str] = Field(None, description="EAID_CA00150C_5B5C_4c32_AE72_A7EE5AEDE015")
    road_name_location_origin_source: Optional[str] = Field(
        None,
        alias="vejnavnebeliggenhed_oprindelse_kilde",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_7F8EB510_AD02_4fd5_844B_C9AB17FB1798.EAID_FD77FC95_714B_4779_B7FA_F9BBAB41758D",
    )
    road_name_location_origin_accuracy_class: Optional[str] = Field(
        None,
        alias="vejnavnebeliggenhed_oprindelse_nøjagtighedsklasse",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_7F8EB510_AD02_4fd5_844B_C9AB17FB1798.EAID_B8A27494_DF27_4aa9_B1F4_EBD68D2479A5",
    )
    road_name_location_origin_registration: Optional[datetime] = Field(
        None,
        alias="vejnavnebeliggenhed_oprindelse_registrering",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_7F8EB510_AD02_4fd5_844B_C9AB17FB1798.EAID_10DCC555_0BE9_4991_AD8C_DEA595CB08A0",
    )
    road_name_location_origin_technical_standard: Optional[str] = Field(
        None,
        alias="vejnavnebeliggenhed_oprindelse_tekniskStandard",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_7F8EB510_AD02_4fd5_844B_C9AB17FB1798.EAID_181528AD_2DA3_4e9d_BF38_81916A924D2A",
    )
    road_name_location_road_name_line: Optional[str] = Field(
        None,
        alias="vejnavnebeliggenhed_vejnavnelinje",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_93C24B4D_0641_4073_B260_1EC7F2024CF4",
    )
    road_name_location_road_name_area: Optional[str] = Field(
        None,
        alias="vejnavnebeliggenhed_vejnavneområde",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_1C496CD9_4541_4574_974D_B641F186077F",
    )
    road_name_location_road_connection_points: Optional[str] = Field(
        None,
        alias="vejnavnebeliggenhed_vejtilslutningspunkter",
        description="EAID_4C594A8E_797B_40bb_B60D_0E5C9159DCD3.EAID_C3E042F2_A66E_40f4_92A1_7C7024A30D16",
    )



from pathlib import Path
from typing import Iterable, Type

from jsonslicer import JsonSlicer
from pydantic import BaseModel

from dar_etl.schemas.address import Adresse
from dar_etl.schemas.address_point import AddressPoint
from dar_etl.schemas.base_model import DarBaseModel
from dar_etl.schemas.house_number import HouseNumber
from dar_etl.schemas.named_road import NamedRoad
from dar_etl.schemas.named_road_municipal_district import NamedRoadMunicipalDistrict
from dar_etl.schemas.named_road_postal_code import NamedRoadPostalCode
from dar_etl.schemas.named_road_supplementary_city_name import NamedRoadSupplementaryCityName
from dar_etl.schemas.postal_code import PostalCode
from dar_etl.schemas.root_keys import Root
from dar_etl.schemas.supplementary_city_name import SupplementaryCityName


class Metadata(BaseModel):
    root: Root
    dar_model: DarBaseModel
    filename: str


class DarParser:
    _registry: dict[Root, Type[DarBaseModel]] = {
        Root.addresses: Adresse,
        Root.address_points: AddressPoint,
        Root.house_numbers: HouseNumber,
        Root.named_roads: NamedRoad,
        Root.named_road_municipal_parts: NamedRoadMunicipalDistrict,
        Root.named_road_postal_codes: NamedRoadPostalCode,
        Root.named_road_supplementary_city_names: NamedRoadSupplementaryCityName,
        Root.postal_codes: PostalCode,
        Root.supplementary_city_names: SupplementaryCityName,
    }

    def parse(self, filepath: Path) -> Iterable[Metadata]:
        with open(file=filepath, mode="r", encoding="UTF-8") as file_pointer:
            for *key, record in JsonSlicer(
                file=file_pointer,
                path_prefix=(None, None),
                read_size=4096,  # noqa: WPS432
                path_mode="full",
            ):
                root = Root(key[0])
                dar_model = self._registry[root](**record)
                yield Metadata(root=root, dar_model=dar_model, filename=filepath.name)



from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class SupplementaryCityName(DarBaseModel):
    name: Optional[str] = Field(None, alias="navn", description="EAID_8359992A_59C4_48c4_981D_7A42F20CA168")
    supplementary_city_name: Optional[str] = Field(
        None,
        alias="supplerendeBynavn",
        description="EAID_C6722E9B_EF6C_425d_9373_A933C01A6A64",
    )



import logging
from pathlib import Path
from typing import Iterable

from confluent_kafka import Consumer

from dar_etl.config import Config


class NewDarFileNotificationConsumer:
    def __init__(self, config: Config) -> None:
        host = config.kafka.host
        port = config.kafka.port

        self.input_dir = config.app.input_directory
        self.consumer = Consumer(
            {
                "bootstrap.servers": f"{host}:{port}",
                "group.id": "dar-etl-notification-consumer",
                "auto.offset.reset": "earliest",
                "max.poll.interval.ms": 86400000,
            },
        )
        self.consumer.subscribe([config.kafka.topic])

    def consume(self) -> Iterable[Path]:
        try:
            while True:  # noqa: WPS457
                yield from self._consume()
        except KeyboardInterrupt:
            logging.info("Consumer was interrupted by user")

    def _consume(self) -> Iterable[Path]:
        for message in self.consumer.consume(timeout=10):
            error = message.error()
            if error:
                logging.error(error)
                continue
            filename: str = message.value().decode("utf-8")
            logging.info(f"[CONSUMED] {filename}")
            yield self.input_dir.joinpath(filename)



from datetime import datetime
from typing import Optional

from pydantic import Field

from dar_etl.schemas.base_model import DarBaseModel


class AddressPoint(DarBaseModel):
    origin_source: Optional[str] = Field(
        None,
        alias="oprindelse_kilde",
        description="EAID_B9AAE6C5_974D_4d20_BABA_C62874774F4B.EAID_FD77FC95_714B_4779_B7FA_F9BBAB41758D",
    )
    origin_accuracy_class: Optional[str] = Field(
        None,
        alias="oprindelse_nøjagtighedsklasse",
        description="EAID_B9AAE6C5_974D_4d20_BABA_C62874774F4B.EAID_B8A27494_DF27_4aa9_B1F4_EBD68D2479A5",
    )
    origin_registration: Optional[datetime] = Field(
        None,
        alias="oprindelse_registrering",
        description="EAID_B9AAE6C5_974D_4d20_BABA_C62874774F4B.EAID_10DCC555_0BE9_4991_AD8C_DEA595CB08A0",
    )
    origin_technical_standard: Optional[str] = Field(
        None,
        alias="oprindelse_tekniskStandard",
        description="EAID_B9AAE6C5_974D_4d20_BABA_C62874774F4B.EAID_181528AD_2DA3_4e9d_BF38_81916A924D2A",
    )
    position: Optional[str] = Field(
        None,
        alias="position",
        description="EAID_4B062EDE_CE8C_414b_AB85_B5A1FD72AA78",
    )



from datetime import datetime
from datetime import timezone
import logging

from confluent_kafka import KafkaError
from confluent_kafka import Message
from confluent_kafka import Producer

from dar_etl.config import Config
from dar_etl.parsing.parser import Metadata
from dar_etl.schemas.base_model import DarBaseModel


class DarEntryPublisher:
    def __init__(self, config: Config) -> None:
        host = config.kafka.host
        port = config.kafka.port
        self.producer = Producer({"bootstrap.servers": f"{host}:{port}"})

    def publish(self, metadata: Metadata) -> None:
        self.producer.poll(0)

        self.producer.produce(
            topic=metadata.root.value,
            key=self._message_key(dar_model=metadata.dar_model),
            value=metadata.dar_model.json().encode(encoding="utf-8"),
            on_delivery=self._on_delivery,
            timestamp=self._message_timestamp(dar_model=metadata.dar_model),
            headers=self._message_headers(metadata=metadata),
        )

    def flush(self) -> None:
        self.producer.flush()

    def _message_headers(self, metadata: Metadata) -> dict[str, bytes]:
        processed_timestamp = datetime.now(tz=timezone.utc).isoformat()
        return {
            "json_file": metadata.filename.encode(encoding="utf-8"),
            "processed_timestamp": processed_timestamp.encode("utf-8"),
        }

    def _message_key(self, dar_model: DarBaseModel) -> bytes | None:
        id_local = dar_model.id_local
        if id_local:
            return id_local.encode(encoding="UTF-8")
        return None

    def _message_timestamp(self, dar_model: DarBaseModel) -> int:
        effect_from = dar_model.effect_from
        if effect_from:
            return int(effect_from.timestamp() * 1000)
        return 0

    def _on_delivery(self, error: KafkaError, message: Message) -> None:
        if error:
            logging.error("Failed to publish.")
        else:
            logging.debug("Published %s successfully", message.key())
